require 'svc_party/version'
require 'svc_party/api_response_handler'
require 'svc_party/api_response'
require 'uri'
require 'json'
require 'httparty'

module SvcParty
  class Client
    include HTTParty
    follow_redirects true

    attr_reader :base_uri

    def initialize(base_uri, verify_ssl = true)
      @base_uri = base_uri
      @verify_ssl = verify_ssl
    end

    def with_get(uri_endpoint, header_data = {})
      response = self.class.get(construct_url(uri_endpoint),
                                verify: @verify_ssl,
                                headers: get_headers(header_data))
      yield ApiResponseHandler.new(response).call

    rescue StandardError => ex
      yield ApiResponseHandler.error(ex)
    end

    def with_post(url, params, header_data = {})
      params = (params.class == String ? params : params.to_json)

      response = self.class.post(construct_url(url),
                                 verify: @verify_ssl,
                                 body: params,
                                 headers: get_headers(header_data))
      yield ApiResponseHandler.new(response).call

    rescue StandardError => ex
      yield ApiResponseHandler.error(ex)
    end

    def with_patch(url, params, header_data = {})
      params = (params.class == String ? params : params.to_json)

      response = self.class.patch(construct_url(url),
                                  verify: @verify_ssl,
                                  body: params,
                                  headers: get_headers(header_data))
      yield ApiResponseHandler.new(response).call

    rescue StandardError => ex
      yield ApiResponseHandler.error(ex)
    end

    def with_delete(url, header_data = {})
      response = self.class.delete(construct_url(url),
                                   verify: @verify_ssl,
                                   headers: get_headers(header_data))
      yield ApiResponseHandler.new(response).call

    rescue StandardError => ex
      yield ApiResponseHandler.error(ex)
    end

    # Extend this in your base class
    def get_headers(_header_data = {})
      raise NotImplementedError
    end

    private

    def construct_url(uri_endpoint)
      # Support APIs that define a follow on url
      return uri_endpoint if uri_endpoint.start_with? 'http'
      URI.join(@base_uri, uri_endpoint)
    end

  end

end
